/*
 Workshop Internet of Things
 Maker Faire Rome 2016
 15-16 ottobre
 STEP 5
 Acquisizione di un ingresso analogico, emissione su uscita analogica,
 controllo di superamento di una soglia ed emissione di un allarme
 diagnostica locale su display LCD con diagnostica sul canale seriale.
 Il loop di programma acquisisce ripetutamente il valore analogico
 dell'ingresso A1 a cui è collegato un sensore di temperatura NTC
 Il valore compreso tra 0 e 1023 viene mappato nel range 0/255
 in modo da poter essere inviato ad una uscita analogica (PWM)
 a cui è collegato un LED che si accende in modo proporzionale al valore letto.
 Il valore grezzo viene convertito in temperatura
 attraverso la formula approssimata di Steinhart-Hart:
 T(°K)= 1/((1/log(R/R0))+(1/T0)
 T(°C)= T(°K)-273.15
 Il valore viene confrontato con una soglia prefissata
 e se la soglia è superata viene emesso un segnale di allarme (LED rosso acceso)
 Il valore convertito viene emesso su un display LCD retroilluminato.
 Il colore di sfondo del display cambia in base allo stato di allarme
 I due valori (originale e scalato) e lo stato di allarme vengono inviati al canale seriale
 Un ritardo di 250 mSec viene inserito per facilitare la visualizzazione.
 
 Hardware:
 * Intel Edison con Edison Kit for Arduino 
 * Grove Starter Kit IoT Edition
 * Grove Temperature Sensor (NTC) 
 * Grove LED Socket Kit (Blu)
 * Grove LED Socket Kit (Rosso)
 * Grove LCD RGB Backlight (I2C)
  
 creato: 05/09/2016
 da Duilio Peroni
 
 http://www.schoolmakerday.it/iot
 Questo codice è di pubblico dominio
 */

//N.B. Il display è collegato al bus I2C
//per comunicare con il display sono necessarie due librerie:
#include <Wire.h>
#include "rgb_lcd.h" 
//per elaborare la formula di Steinhart-Hart è necessaria la liberia matematica
#include <math.h>

//variabili
//defisce l'oggetto lcd per la comunicazione con il display
rgb_lcd lcd;          
//definisce il dosaggio dei colori di sfondo
//da 0 a 255 per ogni colore (0,0,0=nero) - (255,255,255=bianco)
//valori intermedi generano 16 milioni di colori
int red = 255;
int green = 255;
int blue =255;
int valoreAnalogico;    //valore letto dal potenziometro
int uscitaAnalogica;    //valore emesso sul led
char messaggioTemperatura[6];  //rappr. testuale della temperatura
char messaggioLCD[16];  //messaggio inviato al display
float R;                //valore della resistenza del sensore
float temperatura;      //valore della temperatura in °C

//costanti
//N.B. in configurazione di default solo i pin 3,5,6 e 9
//possono essere uscite analogiche
const int pinAnalogico = 3; //definizione del pin di uscita analogica
const int pinAllarme = 2;   //definizione del pin di allarme
const int pinSensore = A1;  //definizione del pin di ingresso analogico
const float soglia = 25.0;  //soglia a temperatura ambiente
//costanti del sensore NTC
const int B = 4275;           //valore B del sensore (pendenza) range 4250/4300
const int R0 = 100000.0;      //valore R0 del sensore (offset=100 Kohm)

// Funzione setup
//  Inizializza il canale seriale 
//  alla velocità di default (9600 bit/s)
//  e imposta D2 e D3 come uscita
void setup() {
  Serial.begin(9600);
  pinMode(pinAnalogico,OUTPUT);
  pinMode(pinAllarme,OUTPUT);
  //configura display
  lcd.begin(16,2);           //formato 16 colonne, due righe
  lcd.setRGB(red,green,blue); //imposta colore di sfondo
  lcd.print("MFRome 2016");  //scrive intestazione su prima riga
}

// Funzione loop
//  viene ripetutamente letto il valore analogico del canale A0
//  viene scalato da 0/1023 a 0/255 per generare il valore di uscita
//  viene inviato all'uscita 
//  (LED blu acceso in modo proporzionale alla rotazione del potenziometro)
//  viene controllata la soglia
//  se il valore supera la soglia il LED rosso viene acceso altrimenti viene spento
//  Il valore scalato viene emesso su un display LCD retroilluminato.
//  Il colore di sfondo del display cambia in base allo stato di allarme
//  il risultato viene inviato sul canale seriale per diagnostica
//  un ritardo di 250 mSec facilita la lettura del monitor
void loop() {
  //legge, mette in scala ed emette in uscita
  valoreAnalogico=analogRead(pinSensore);             //legge ingresso analogico
  uscitaAnalogica=map(valoreAnalogico,0,1023,0,255);  //mette in scala
  analogWrite(pinAnalogico,uscitaAnalogica);          //scrive uscita analogica  
  //invia valori su seriale per diagnostica
  Serial.print(valoreAnalogico);                      //invia per diagnostica
  Serial.print(" "); 
  Serial.print(uscitaAnalogica);
  //converte il dato grezzo in misura della temperatura (°C)
  //partizione di tensione in ingresso (R1 = 1 kohm)
  R = 1023.0/((float)valoreAnalogico)-1.0; 
  R = 100000.0*R;
  //formula di Steinerhart-Hart
  temperatura=1.0/(log(R/R0)/B+1/298.15)-273.15;
  Serial.print(" T: ");
  Serial.print(temperatura);
  //converte valore in messaggio per diplay
  dtostrf(temperatura, 4, 2, messaggioTemperatura);
  sprintf(messaggioLCD,"%s C", messaggioTemperatura);
  lcd.setCursor(0,1);     //posizione prima colonna, seconda riga
  lcd.print(messaggioLCD);
  //verifica la soglia
  if(temperatura > soglia) {  //allarme ON
    digitalWrite(pinAllarme,HIGH);  
    red=255; green=0; blue=0;       //sondo rosso
    lcd.setRGB(red,green,blue);
    Serial.println(" Allarme ON");
  }
  else {                        //allarme OFF
    digitalWrite(pinAllarme,LOW);      
    red=0; green=255; blue=0;       //sondo verde
    lcd.setRGB(red,green,blue);
    Serial.println(" Allarme OFF");
  }
  delay(250);                                         //attende 
}

//conversione da float a stringa
char *dtostrf (double val, signed char width, unsigned char prec, char *sout) {
  uint32_t iPart = (uint32_t)val;
  uint32_t dPart = (uint32_t)((val - (double)iPart) * pow(10, prec));
  sprintf(sout, "%02d.%02d", iPart, dPart);  
  return sout;
}

